% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% % Implements the First order DLM in Example 1 of 
% % Carvalho et al, Particle Learning and Smoothing, Stat. Sci., 
% % vol. 25, no. 1, pp. 88-106, 2010
% linear Gaussian model
% X_{n+1} = X_n+tau* V_{n+1},   v_{n+1} ~ N(0,1) , x_0 ~ N(m0,C0)
% Y_n = rho* X_n + sigma*W_n,  W_n ~ N(0,1)
% 
% NOTE: tau2 known
% 
% This script stores all the history of the particles
% and weights, but only current sufficient statistics 
% 
% Can run for different priors on theta, uniform, normal and truncated
% gaussian
% 
% Authors: N. Chopin, N. Kantas, S.S. Singh, A. Doucet
% Date: November 2011



% INIT
clear
tic

N = 10000;   % Number of particles
change_data = false;
name_file = 'simulated_y.mat';

T = 5000;   % Number of time steps.


if change_data | (~exist(name_file))
    true_x = zeros(T,1);        % Hidden states.
    y = zeros(T,1);        % Observations.
    true_rho = 0.5;
    true_sigma=sqrt(1);
    tau2=.1;
    %     % GENERATE TRUE STATE AND MEASUREMENTS
    true_x(1) = randn; % because x0=0
    for t=2:T,
        true_x(t) = true_rho*true_x(t-1) + sqrt(tau2)*randn;
    end;
    y = true_x+true_sigma*randn(T,1);
    save(name_file,'y','true_x','true_sigma','true_rho','tau2')
else
    load(name_file,'y','tau2','true_rho')
end

T=size(y,1);

% Particle learning

gama=1;
gamb=1;
rho=zeros(N,T);
x=zeros(N,T);
sigmay2=ones(N,T);
% SAMPLE FROM THE PRIOR:n
x(:,1) = zeros(N,1); % because x0=0 a.s.

prior_type='uniform';
prior_rho_mean=true_rho;
prior_rho_var=tau2;
switch lower(prior_type)
    case {'normal','gaussian'}
        rho(:,1)=prior_rho_mean*ones(N,1)+sqrt(tau2)*randn(N,1);
    case 'truncated gaussian'
        for n=1:N
            reject =1;
            while reject
                temp=prior_rho_mean + sqrt(prior_rho_var)*randn;
                reject = (abs(temp)>1);
            end
            rho(n,1)=temp;
        end
    case 'uniform'
        rho(:,1)=-1+2*rand(N,1);
    otherwise
        disp('Unknown prior.')
end

sigmay2(:,1)=1./(gamrnd( gama,1/gamb,N,1));% 0.01*ones(N,1);%
% Suff Statistics
suf1 = zeros(N,1);% x_t times x_t+1
suf2 = zeros(N,1);% x_t^2
suf3 = zeros(N,1);% (y_t-x_t)^2
% normalised weights
w = zeros(N,T);
% unormalised weights
uw = zeros(N,1);

for t=1:T
   
  
        m = rho(:,t).*x(:,t); %mean
        q = 1./(tau2+sigmay2(:,t)); %precision

        uw = sqrt(q).*exp(-0.5.*q.*((m-repmat(y(t),N,1)).^2)); % NC: MULTIPLY BY 1/SIGMAY !

        w(:,t) = uw/sum(uw);

        % resample from p(y+1|xt)
        Res= multinomialR([1:N]',w(:,t));

        x(:,t) = x(Res,t);
        rho(:,t) = rho(Res,t);
        sigmay2(:,t) = sigmay2(Res,t); % NC: ADDED THIS
        suf1= suf1(Res);
        suf2= suf2(Res);
        suf3= suf3(Res);

        % propose new process state particle
        s_sqrd=1./ (1/tau2 + 1./sigmay2(:,t));
        propMean=s_sqrd.* (rho(:,t).*x(:,t)/tau2  + y(t)./sigmay2(:,t)  );
        x(:,t+1) =propMean+sqrt(s_sqrd).*randn(N,1);
        % NOTE: from now on, xu now is particle approx of p(x_t|y_1:t),
        % ie for next iteration for next iteration

        % propose new parameter particle
        suf1 = suf1+x(:,t+1).*x(:,t); % NOTE: x(:,t) is x_t-1, xu is x_t
        suf2 = suf2+x(:,t).^2; % NOTE: which means (x_t-1)^2
        suf3 = suf3+(x(:,t+1)-y(t)).^2; % NC: CHANGED THIS!


        switch lower(prior_type)
            case {'normal','gaussian'}
                sigrho=1./(suf2/tau2+1/prior_rho_var);
                mrho=sigrho.*(prior_rho_mean/prior_rho_var+suf1/tau2);
                rho(:,t+1)=mrho + sqrt(sigrho)*randn;
            case 'uniform'
                % do accept/reject to sample p( rho_t | x_{1:t},y_{1:t})

                if t==1
                    % p(rho|y1) = prior(rho)=U[-1,1] because x0 = 0
                    rho(:,t+1) = -1+2*rand(N,1);
                else
                    for n=1:N
                        mrho=suf1(n)/suf2(n);
                        sigrho=sqrt(tau2/suf2(n));
                        reject =1;
                        while reject
                            temp=mrho + sigrho*randn;
                            reject = (abs(temp)>1);
                        end
                        rho(n,t+1)=temp;
                    end
                end
            case 'truncated gaussian'
                for n=1:N
                    sigrho=1/(suf2(n)/tau2+1/prior_rho_var);
                    mrho=sigrho*(suf1(n)/tau2+prior_rho_mean/prior_rho_var);
                    reject =1;
                    while reject
                        temp=mrho + sqrt(sigrho)*randn;
                        reject = (abs(temp)>1);
                    end
                    rho(n,t+1)=temp;
                end
            otherwise
                disp('Unknown prior.')
        end

        % sample sigmay or d
        sigmay2(:,t+1)=1./(  gamrnd((gama+t/2)*ones(N,1) ,1./(gamb+0.5*suf3))   );
        % NC: This must be sigmay2u not sigmay2
end

clear temp
simtime=toc
