% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% % Implements the First order DLM in Example 1 of 
% % Carvalho et al, Particle Learning and Smoothing, Stat. Sci., 
% % vol. 25, no. 1, pp. 88-106, 2010
% linear Gaussian model
% X_{n+1} = X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) , x_0 ~ N(m0,C0)
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
% 
% This script stores all the history of the particles, sufficient
% statistics and weights
% 
% 
% Authors:  N. Kantas, A. Doucet, S.S. Singh, N. Chopin
% Date: November 2011


clear;
tic;

N =10000; % number of particles


% True parameters and true initial state for generating observations
sigma=sqrt(1);
tau=sqrt(1);

change_data = false; 
name_file =  'simulated_y_dlm.mat';

if change_data | (~exist(name_file))
    T = 10000; % Number of time steps
    x0 = 0;              
    % GENERATE TRUE STATE AND MEASUREMENTS% Generate data record
    x=tau*randn(1,T);
    x(1)= x(1) + x0;
    x=cumsum(x);
    y=x+sigma*randn(1,T);


    save(name_file,'y')
else
    load(name_file,'y')
end

T=size(y,2);


% Declare priors
m0=0; C0=10; % x0 is N(m0,C0)
a0=1; b0=1; % sigma2 is IG(a0,b0)
c0=1; d0=1; % tau2 is IG(c0,d0)

% Declare variables for storing particles

xHat=zeros(N,T+1);  % particle approximation of p(x_{t-1}|y_{1:t})
sigma2Hat=zeros(N,T+1);
tau2Hat=zeros(N,T+1);
suf1Hat=zeros(N,T+1);
suf2Hat=zeros(N,T+1);
w=zeros(N,1); % normalised weights

% Initialise particles by sampling from priors
xHat(:,1)=m0+sqrt(C0)*randn(N,1);
sigma2Hat(:,1) = 1./ gamrnd(a0,1/b0,N,1);
tau2Hat(:,1) = 1./ gamrnd(c0,1/d0,N,1);


for t=1:T
    
    % compute weights for resampling
    w=normpdf( y(t)*ones(N,1),xHat(:,t), sqrt(sigma2Hat(:,t)+tau2Hat(:,t)) );
    w=w/sum(w);
    
    % resample with weights prop to  from p(yt|xt-1,simga2,tau2)
    index= multinomialR([1:N]',w);
    xHat(:,t)=xHat(index,t);                        % now xHat(:,t) is 
                                                    % the particle approx
                                                    % of p(x_{t-1} | y_t)
    sigma2Hat(:,t)=sigma2Hat(index,t);              
    tau2Hat(:,t)=tau2Hat(index,t);
    
    suf1Hat(:,t)=suf1Hat(index,t);
    suf2Hat(:,t)=suf2Hat(index,t);
    
    
    % propagate state samples
    var_temp=1./( 1./sigma2Hat(:,t) + 1./tau2Hat(:,t) );
    mean_temp=var_temp.* ( y(t)./sigma2Hat(:,t) + xHat(:,t)./tau2Hat(:,t) );
    x_temp=mean_temp+sqrt(var_temp).*randn(N,1); % now x_temp is 
                                                    % the particle approx of p(x_t | y_t)
    suf1Hat(:,t+1)=suf1Hat(:,t)+(x_temp-xHat(:,t)).^2;
    suf2Hat(:,t+1)=suf2Hat(:,t)+(y(t)-x_temp).^2;
    xHat(:,t+1)=x_temp;  % xHat(:,t+1) now contains the particle approx of p(x_t | y_t)
    
    % update model parameter samples   
    sigma2Hat(:,t+1)= 1./gamrnd(a0*ones(N,1)+t/2,1./(b0+suf2Hat(:,t+1)/2)  ); % sigma2Hat(:,t+1) now contains the particle approx of p(sigma2_t | y_t)
    tau2Hat(:,t+1)= 1./gamrnd(c0*ones(N,1)+t/2,1./(d0+suf1Hat(:,t+1)/2)  );
    
    
end



simtime=toc
