% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% % Implements the First order DLM in Example 1 of 
% % Carvalho et al, Particle Learning and Smoothing, Stat. Sci., 
% % vol. 25, no. 1, pp. 88-106, 2010
% linear Gaussian model
% X_{n+1} = X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) , x_0 ~ N(m0,C0)
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
% 
% This script stores only full history of the particles containing sigma^2,
% tau^2 and X_{0:t}  
% Can be used for memory intensive simulations
% 
% Authors:  N. Kantas, N. Chopin, S.S. Singh, A. Doucet
% Date: November 2011

clear;
tic;

N =10000; % number of particles

GetData;

T=size(y,2);

% Declare priors
m0=0; C0=10; % x0 is N(m0,C0)
a0=1; b0=1; % sigma2 is IG(a0,b0)
c0=1; d0=1; % tau2 is IG(c0,d0)


% Declare variables for storing particles

xHat=zeros(N,T+1);  % particle approximation of p(x_{t-1}|y_{1:t})
sigma2Hat=zeros(N,T+1);
tau2Hat=zeros(N,T+1);
suf1Hat=zeros(N,1);
suf2Hat=zeros(N,1);
w=zeros(N,1); % normalised weights

% Initialise particles by sampling from priors
xHat(:,1)=m0+sqrt(C0)*randn(N,1);
sigma2Hat(:,1) = 1./ gamrnd(a0,1/b0,N,1);
tau2Hat(:,1) = 1./ gamrnd(c0,1/d0,N,1);


for t=1:T
    
    % compute weights for resampling
    w=normpdf( y(t)*ones(N,1),xHat(:,t), sqrt(sigma2Hat(:,t)+tau2Hat(:,t)) );
    w=w/sum(w);
    
    % resample with weights prop to  from p(yt|xt-1,simga2,tau2)
    index= multinomialR([1:N]',w);
    xHat(:,t)=xHat(index,t);                        % now xHat(:,t) is 
                                                    % the particle approx
                                                    % of p(x_{t-1} | y_t)
    sigma2Hat(:,t)=sigma2Hat(index,t);              
    tau2Hat(:,t)=tau2Hat(index,t);
    
    suf1Hat(:,:)=suf1Hat(index,:);
    suf2Hat(:,:)=suf2Hat(index,:);
    
    
    % propagate state samples
    var_temp=1./( 1./sigma2Hat(:,t) + 1./tau2Hat(:,t) );
    mean_temp=var_temp.* ( y(t)./sigma2Hat(:,t) + xHat(:,t)./tau2Hat(:,t) );
    x_temp=mean_temp+sqrt(var_temp).*randn(N,1); % now x_temp is 
                                                    % the particle approx of p(x_t | y_t)
    suf1Hat(:,:)=suf1Hat(:,:)+(x_temp-xHat(:,t)).^2;
    suf2Hat(:,:)=suf2Hat(:,:)+(y(t)-x_temp).^2;
    xHat(:,t+1)=x_temp;  % xHat(:,t+1) now contains the particle approx of p(x_t | y_t)
    
    % update model parameter samples   
    sigma2Hat(:,t+1)= 1./gamrnd(a0*ones(N,1)+t/2,1./(b0+suf2Hat(:,:)/2)  ); % sigma2Hat(:,t+1) now contains the particle approx of p(sigma2_t | y_t)
    tau2Hat(:,t+1)= 1./gamrnd(c0*ones(N,1)+t/2,1./(d0+suf1Hat(:,:)/2)  );
    
    
end



simtime=toc


